/*-----------------------------------------------------------------------

File  : ast.h

Author: Stephan Schulz (schulz@eprover.org)

Contents
 

  Copyright 2015 by the author.
  This code is released under the GNU General Public Licence.

Changes

<1> Tue Mar 31 16:25:43 CEST 2015
    New

-----------------------------------------------------------------------*/

#ifndef AST

#define AST

#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <assert.h>


/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef enum
{
   nil,
   prog,
   def,
   vardef,
   fundef,
   type,
   params,
   paramlist,
   param,
   body,
   vardefs,
   stmts,
   stmt,
   while_stmt,
   if_stmt,
   ret_stmt,
   print_stmt,
   assign,
   funcall_stmt,
   boolexpr,
   expr,
   funcall,
   args,
   arglist,
   arg,
   t_MULT,
   t_DIV,
   t_PLUS,
   t_MINUS,
   t_EQ,
   t_NEQ,
   t_LT, 
   t_GT, 
   t_LEQ, 
   t_GEQ,
   t_IDENT,
   t_STRINGLIT,
   t_INTLIT,
   t_INTEGER,
   t_STRING,
   t_IF,
   t_WHILE,
   t_RETURN,
   t_PRINT,
   t_dummy
}ASTNodeType;



typedef struct astcell
{
   ASTNodeType type;
   long        nodectr;
   int         line;
   int         col;
   char*       litval;
   long        intval;
   struct astcell *child1;
   struct astcell *child2;
   struct astcell *child3;
   struct astcell *child4;
}ASTCell, *AST_p;



/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

extern long nodectr;

#define ASTCellAlloc()    (ASTCell*)malloc(sizeof(ASTCell))
#define ASTCellFree(junk) free(junk)

AST_p  ASTAlloc(ASTNodeType type, int line, int col, char* litval, 
                long intval, AST_p child1, AST_p child2, AST_p child3, 
                AST_p child4);

#define ASTAlloc2(type, line, col, litval, intval, child1, child2) \
   ASTAlloc((type), (line), (col), (litval), (intval), (child1), (child2), NULL, NULL)

void   ASTFree(AST_p junk);

void   DOTASTNodePrint(FILE* out, AST_p ast);
void   DOTASTPrint(FILE* out, AST_p ast);

void   ExprASTPrint(FILE* out, AST_p ast);
void   SExprASTPrint(FILE* out, AST_p ast);


#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





