 /* -*- Mode: c -*-

     Lexer for a minimal programming language "nanoLang"

     Copyright 2015 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

%option noyywrap
%option yylineno

%{
   #include <string.h>
   #include "ast.h"
   #include "nanoparse.tab.h"   

   int ret_tok(int tok)
   {
      return tok;
   }

   int yycol = 0;

#define YY_USER_ACTION {yycol += yyleng;}

%}

DIGIT      [0-9]
ALPHA      [a-zA-Z]
ALNUM      {ALPHA}|{DIGIT}

INT        {DIGIT}+
ID         {ALPHA}{ALNUM}*


%%

"("  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(OPENPAR);}
")"  {yylval = ASTAlloc2(t_dummy, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(CLOSEPAR);}
"*"  {yylval = ASTAlloc2(t_MULT, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(MULT);}
"/"  {yylval = ASTAlloc2(t_DIV, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(DIV);}
"+"  {yylval = ASTAlloc2(t_PLUS, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(PLUS);}
"-"  {yylval = ASTAlloc2(t_MINUS, yylineno, yycol-yyleng, NULL, 0, NULL, NULL);
      return ret_tok(MINUS);}

       
{ALPHA}{ALNUM}* {yylval = ASTAlloc2(t_IDENT, yylineno, yycol-yyleng, 
                                   strdup(yytext), 0, NULL, NULL);
                 return ret_tok(IDENT);}

\"[^"]*\"       {yylval = ASTAlloc2(t_STRINGLIT, yylineno, yycol-yyleng, 
                                   strdup(yytext), 0, NULL, NULL);
                 return ret_tok(STRINGLIT);}

{DIGIT}+        {yylval = ASTAlloc2(t_INTLIT, yylineno, yycol-yyleng, 
                                   strdup(yytext), atoll(yytext), NULL, NULL);
                 return ret_tok(INTLIT);}


#[^\n]*\n { /* Eat comments */}

[\n]    {yycol = 0;}

[\r\t ] { /* Skip  whitespace*/ }


 /* Everything else is an invalid character. */
.   {return ret_tok(ERROR);}

%%

