 /* 
     nanoLang expression parser

     Copyright 2015 by Stephan Schulz, schulz@eprover.org.

     This code is released under the GNU General Public Licence
     Version 2.
 */

%{
   #include <stdio.h>
   #include <string.h>
   #include "ast.h"
   
   extern int yyerror(const char* err);
   extern int yylex(void);
   extern FILE *yyin;
   

   AST_p ast;

   /* #define YYSTYPE AST_p */
%}

%define api.value.type {AST_p}

%define parse.error verbose

%start start

%token IDENT
%token STRINGLIT
%token INTLIT
%left  PLUS MINUS
%left  MULT DIV
%right UMINUS
%token OPENPAR CLOSEPAR

%token ERROR


%%

start: expr {$$=$1; ast=$1;}

expr: INTLIT  {$$ = $1;}
    | IDENT   {$$ = $1;}
    | STRINGLIT {$$ = $1;}
    | OPENPAR expr CLOSEPAR {$$ = $2;}
    | expr PLUS expr  {$$=ASTAlloc2(t_PLUS, $1->line, $1->col, NULL, 0, $1, $3);}
    | expr MINUS expr {$$=ASTAlloc2(t_MINUS, $1->line, $1->col, NULL, 0, $1, $3);}
    | expr MULT expr  {$$=ASTAlloc2(t_MULT, $1->line, $1->col, NULL, 0, $1, $3);}
    | expr DIV expr   {$$=ASTAlloc2(t_DIV, $1->line, $1->col, NULL, 0, $1, $3);}
    | MINUS expr %prec UMINUS {$$=ASTAlloc2(t_MINUS, $1->line, $1->col, NULL, 0, $2, NULL);} 
;



%%


int yyerror(const char* err)
{
   printf("Error: %s\n", err);
   return 0;
}


int main (int argc, char* argv[])
{
  int i;
  int res;
  bool printdot   = false;
  bool printexpr  = true;
  bool printsexpr = false;

   ++argv, --argc;  /* skip over program name */

   if (argc > 0)
   {
      if(strcmp(argv[0], "--dot")==0)
      {
         printdot   = true;
         printexpr  = false;
         printsexpr = false;         
         ++argv, --argc;
      }
      else if(strcmp(argv[0], "--sexpr")==0)
      {
         printdot   = false;
         printexpr  = false;
         printsexpr = true;         
         ++argv, --argc;
      }
   }
   
   if ( argc > 0 )
   {
      yyin = fopen( argv[0], "r" );
   }
   else
   {
      yyin = stdin;
   }   

   res = yyparse();

   if(printdot)
   {
      DOTASTPrint(stdout, ast);
   }
   if(printexpr)
   {
      ExprASTPrint(stdout, ast);
      printf("\n");      
   }
   if(printsexpr)
   {
      SExprASTPrint(stdout, ast);
      printf("\n");
   }
   return res;
}
